@@ -1,38 +1,31 @@
-"""
-    sphinx.ext.autodoc.type_comment
-    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
+"""Update annotations info of living objects using type_comments."""
 
-    Update annotations info of living objects using type_comments.
-
-    :copyright: Copyright 2007-2021 by the Sphinx team, see AUTHORS.
-    :license: BSD, see LICENSE for details.
-"""
+from __future__ import annotations
 
+import ast
 from inspect import Parameter, Signature, getsource
-from typing import Any, Dict, List, cast
+from typing import Any, cast
 
 import sphinx
 from sphinx.application import Sphinx
 from sphinx.locale import __
-from sphinx.pycode.ast import ast
-from sphinx.pycode.ast import parse as ast_parse
 from sphinx.pycode.ast import unparse as ast_unparse
 from sphinx.util import inspect, logging
 
 logger = logging.getLogger(__name__)
 
 
-def not_suppressed(argtypes: List[ast.AST] = []) -> bool:
+def not_suppressed(argtypes: list[ast.AST] = []) -> bool:
     """Check given *argtypes* is suppressed type_comment or not."""
     if len(argtypes) == 0:  # no argtypees
         return False
-    elif len(argtypes) == 1 and ast_unparse(argtypes[0]) == "...":  # suppressed
+    if len(argtypes) == 1 and ast_unparse(argtypes[0]) == "...":  # suppressed
         # Note: To support multiple versions of python, this uses ``ast_unparse()`` for
         # comparison with Ellipsis.  Since 3.8, ast.Constant has been used to represent
         # Ellipsis node instead of ast.Ellipsis.
         return False
-    else:  # not suppressed
-        return True
+    # not suppressed
+    return True
 
 
 def signature_from_ast(node: ast.FunctionDef, bound_method: bool,
@@ -42,10 +35,9 @@ def signature_from_ast(node: ast.FunctionDef, bound_method: bool,
     :param bound_method: Specify *node* is a bound method or not
     """
     params = []
-    if hasattr(node.args, "posonlyargs"):  # for py38+
-        for arg in node.args.posonlyargs:  # type: ignore
-            param = Parameter(arg.arg, Parameter.POSITIONAL_ONLY, annotation=arg.type_comment)
-            params.append(param)
+    for arg in node.args.posonlyargs:
+        param = Parameter(arg.arg, Parameter.POSITIONAL_ONLY, annotation=arg.type_comment)
+        params.append(param)
 
     for arg in node.args.args:
         param = Parameter(arg.arg, Parameter.POSITIONAL_OR_KEYWORD,
@@ -84,25 +76,25 @@ def signature_from_ast(node: ast.FunctionDef, bound_method: bool,
         return Signature(params)
 
 
-def get_type_comment(obj: Any, bound_method: bool = False) -> Signature:
+def get_type_comment(obj: Any, bound_method: bool = False) -> Signature | None:
     """Get type_comment'ed FunctionDef object from living object.
 
     This tries to parse original code for living object and returns
-    Signature for given *obj*.  It requires py38+ or typed_ast module.
+    Signature for given *obj*.
     """
     try:
         source = getsource(obj)
         if source.startswith((' ', r'\t')):
             # subject is placed inside class or block.  To read its docstring,
             # this adds if-block before the declaration.
-            module = ast_parse('if True:\n' + source)
+            module = ast.parse('if True:\n' + source, type_comments=True)
             subject = cast(ast.FunctionDef, module.body[0].body[0])  # type: ignore
         else:
-            module = ast_parse(source)
-            subject = cast(ast.FunctionDef, module.body[0])  # type: ignore
+            module = ast.parse(source, type_comments=True)
+            subject = cast(ast.FunctionDef, module.body[0])
 
         if getattr(subject, "type_comment", None):
-            function = ast_parse(subject.type_comment, mode='func_type')
+            function = ast.parse(subject.type_comment, mode='func_type', type_comments=True)
             return signature_from_ast(subject, bound_method, function)  # type: ignore
         else:
             return None
@@ -133,7 +125,7 @@ def update_annotations_using_type_comments(app: Sphinx, obj: Any, bound_method:
         logger.warning(__("Failed to parse type_comment for %r: %s"), obj, exc)
 
 
-def setup(app: Sphinx) -> Dict[str, Any]:
+def setup(app: Sphinx) -> dict[str, Any]:
     app.connect('autodoc-before-process-signature', update_annotations_using_type_comments)
 
     return {'version': sphinx.__display_version__, 'parallel_read_safe': True}
