@@ -1,32 +1,24 @@
-"""
-    sphinx.ext.linkcode
-    ~~~~~~~~~~~~~~~~~~~
+"""Add external links to module code in Python object descriptions."""
 
-    Add external links to module code in Python object descriptions.
+from __future__ import annotations
 
-    :copyright: Copyright 2007-2019 by the Sphinx team, see AUTHORS.
-    :license: BSD, see LICENSE for details.
-"""
+from typing import Any
 
 from docutils import nodes
+from docutils.nodes import Node
 
 import sphinx
 from sphinx import addnodes
+from sphinx.application import Sphinx
 from sphinx.errors import SphinxError
 from sphinx.locale import _
 
-if False:
-    # For type annotation
-    from typing import Any, Dict, Set  # NOQA
-    from sphinx.application import Sphinx  # NOQA
-
 
 class LinkcodeError(SphinxError):
     category = "linkcode error"
 
 
-def doctree_read(app, doctree):
-    # type: (Sphinx, nodes.Node) -> None
+def doctree_read(app: Sphinx, doctree: Node) -> None:
     env = app.builder.env
 
     resolve_target = getattr(env.config, 'linkcode_resolve', None)
@@ -41,9 +33,9 @@ def doctree_read(app, doctree):
         'js': ['object', 'fullname'],
     }
 
-    for objnode in doctree.traverse(addnodes.desc):
+    for objnode in list(doctree.findall(addnodes.desc)):
         domain = objnode.get('domain')
-        uris = set()  # type: Set[str]
+        uris: set[str] = set()
         for signode in objnode:
             if not isinstance(signode, addnodes.desc_signature):
                 continue
@@ -75,8 +67,7 @@ def doctree_read(app, doctree):
             signode += onlynode
 
 
-def setup(app):
-    # type: (Sphinx) -> Dict[str, Any]
+def setup(app: Sphinx) -> dict[str, Any]:
     app.connect('doctree-read', doctree_read)
     app.add_config_value('linkcode_resolve', None, '')
     return {'version': sphinx.__display_version__, 'parallel_read_safe': True}
