@@ -1,18 +1,12 @@
-"""
-    sphinx.util.logging
-    ~~~~~~~~~~~~~~~~~~~
+"""Logging utility functions for Sphinx."""
 
-    Logging utility functions for Sphinx.
-
-    :copyright: Copyright 2007-2021 by the Sphinx team, see AUTHORS.
-    :license: BSD, see LICENSE for details.
-"""
+from __future__ import annotations
 
 import logging
 import logging.handlers
 from collections import defaultdict
 from contextlib import contextmanager
-from typing import IO, TYPE_CHECKING, Any, Dict, Generator, List, Optional, Tuple, Type, Union
+from typing import IO, TYPE_CHECKING, Any, Generator
 
 from docutils import nodes
 from docutils.nodes import Node
@@ -20,6 +14,7 @@ from docutils.utils import get_source_line
 
 from sphinx.errors import SphinxWarning
 from sphinx.util.console import colorize
+from sphinx.util.osutil import abspath
 
 if TYPE_CHECKING:
     from sphinx.application import Sphinx
@@ -28,8 +23,7 @@ if TYPE_CHECKING:
 NAMESPACE = 'sphinx'
 VERBOSE = 15
 
-LEVEL_NAMES: Dict[str, int] = defaultdict(lambda: logging.WARNING)
-LEVEL_NAMES.update({
+LEVEL_NAMES: defaultdict[str, int] = defaultdict(lambda: logging.WARNING, {
     'CRITICAL': logging.CRITICAL,
     'SEVERE': logging.CRITICAL,
     'ERROR': logging.ERROR,
@@ -39,22 +33,20 @@ LEVEL_NAMES.update({
     'DEBUG': logging.DEBUG,
 })
 
-VERBOSITY_MAP: Dict[int, int] = defaultdict(lambda: 0)
-VERBOSITY_MAP.update({
+VERBOSITY_MAP: defaultdict[int, int] = defaultdict(lambda: logging.NOTSET, {
     0: logging.INFO,
     1: VERBOSE,
     2: logging.DEBUG,
 })
 
-COLOR_MAP = defaultdict(lambda: 'blue',
-                        {
-                            logging.ERROR: 'darkred',
-                            logging.WARNING: 'red',
-                            logging.DEBUG: 'darkgray'
-                        })
+COLOR_MAP: defaultdict[int, str] = defaultdict(lambda: 'blue', {
+    logging.ERROR: 'darkred',
+    logging.WARNING: 'red',
+    logging.DEBUG: 'darkgray',
+})
 
 
-def getLogger(name: str) -> "SphinxLoggerAdapter":
+def getLogger(name: str) -> SphinxLoggerAdapter:
     """Get logger wrapped by :class:`sphinx.util.logging.SphinxLoggerAdapter`.
 
     Sphinx logger always uses ``sphinx.*`` namespace to be independent from
@@ -76,7 +68,7 @@ def getLogger(name: str) -> "SphinxLoggerAdapter":
     return SphinxLoggerAdapter(logger, {})
 
 
-def convert_serializable(records: List[logging.LogRecord]) -> None:
+def convert_serializable(records: list[logging.LogRecord]) -> None:
     """Convert LogRecord serializable."""
     for r in records:
         # extract arguments to a message and clear them
@@ -85,7 +77,7 @@ def convert_serializable(records: List[logging.LogRecord]) -> None:
 
         location = getattr(r, 'location', None)
         if isinstance(location, nodes.Node):
-            r.location = get_node_location(location)  # type: ignore
+            r.location = get_node_location(location)
 
 
 class SphinxLogRecord(logging.LogRecord):
@@ -97,7 +89,7 @@ class SphinxLogRecord(logging.LogRecord):
         message = super().getMessage()
         location = getattr(self, 'location', None)
         if location:
-            message = '%s: %s%s' % (location, self.prefix, message)
+            message = f'{location}: {self.prefix}{message}'
         elif self.prefix not in message:
             message = self.prefix + message
 
@@ -111,14 +103,23 @@ class SphinxInfoLogRecord(SphinxLogRecord):
 
 class SphinxWarningLogRecord(SphinxLogRecord):
     """Warning log record class supporting location"""
-    prefix = 'WARNING: '
+    @property
+    def prefix(self) -> str:  # type: ignore
+        if self.levelno >= logging.CRITICAL:
+            return 'CRITICAL: '
+        elif self.levelno >= logging.ERROR:
+            return 'ERROR: '
+        else:
+            return 'WARNING: '
 
 
 class SphinxLoggerAdapter(logging.LoggerAdapter):
     """LoggerAdapter allowing ``type`` and ``subtype`` keywords."""
     KEYWORDS = ['type', 'subtype', 'location', 'nonl', 'color', 'once']
 
-    def log(self, level: Union[int, str], msg: str, *args: Any, **kwargs: Any) -> None:
+    def log(  # type: ignore[override]
+        self, level: int | str, msg: str, *args: Any, **kwargs: Any,
+    ) -> None:
         if isinstance(level, int):
             super().log(level, msg, *args, **kwargs)
         else:
@@ -128,7 +129,7 @@ class SphinxLoggerAdapter(logging.LoggerAdapter):
     def verbose(self, msg: str, *args: Any, **kwargs: Any) -> None:
         self.log(VERBOSE, msg, *args, **kwargs)
 
-    def process(self, msg: str, kwargs: Dict) -> Tuple[str, Dict]:  # type: ignore
+    def process(self, msg: str, kwargs: dict) -> tuple[str, dict]:  # type: ignore
         extra = kwargs.setdefault('extra', {})
         for keyword in self.KEYWORDS:
             if keyword in kwargs:
@@ -163,7 +164,7 @@ class NewLineStreamHandler(logging.StreamHandler):
 class MemoryHandler(logging.handlers.BufferingHandler):
     """Handler buffering all logs."""
 
-    buffer: List[logging.LogRecord]
+    buffer: list[logging.LogRecord]
 
     def __init__(self) -> None:
         super().__init__(-1)
@@ -171,6 +172,11 @@ class MemoryHandler(logging.handlers.BufferingHandler):
     def shouldFlush(self, record: logging.LogRecord) -> bool:
         return False  # never flush
 
+    def flush(self) -> None:
+        # suppress any flushes triggered by importing packages that flush
+        # all handlers at initialization time
+        pass
+
     def flushTo(self, logger: logging.Logger) -> None:
         self.acquire()
         try:
@@ -180,14 +186,14 @@ class MemoryHandler(logging.handlers.BufferingHandler):
         finally:
             self.release()
 
-    def clear(self) -> List[logging.LogRecord]:
+    def clear(self) -> list[logging.LogRecord]:
         buffer, self.buffer = self.buffer, []
         return buffer
 
 
 @contextmanager
 def pending_warnings() -> Generator[logging.Handler, None, None]:
-    """Contextmanager to pend logging warnings temporary.
+    """Context manager to postpone logging warnings temporarily.
 
     Similar to :func:`pending_logging`.
     """
@@ -215,7 +221,7 @@ def pending_warnings() -> Generator[logging.Handler, None, None]:
 
 @contextmanager
 def suppress_logging() -> Generator[MemoryHandler, None, None]:
-    """Contextmanager to suppress logging all logs temporary.
+    """Context manager to suppress logging all logs temporarily.
 
     For example::
 
@@ -244,7 +250,7 @@ def suppress_logging() -> Generator[MemoryHandler, None, None]:
 
 @contextmanager
 def pending_logging() -> Generator[MemoryHandler, None, None]:
-    """Contextmanager to pend logging all logs temporary.
+    """Context manager to postpone logging all logs temporarily.
 
     For example::
 
@@ -264,7 +270,7 @@ def pending_logging() -> Generator[MemoryHandler, None, None]:
 
 @contextmanager
 def skip_warningiserror(skip: bool = True) -> Generator[None, None, None]:
-    """contextmanager to skip WarningIsErrorFilter for a while."""
+    """Context manager to skip WarningIsErrorFilter temporarily."""
     logger = logging.getLogger(NAMESPACE)
 
     if skip is False:
@@ -284,7 +290,7 @@ def skip_warningiserror(skip: bool = True) -> Generator[None, None, None]:
 
 @contextmanager
 def prefixed_warnings(prefix: str) -> Generator[None, None, None]:
-    """Prepend prefix to all records for a while.
+    """Context manager to prepend prefix to all warning log records temporarily.
 
     For example::
 
@@ -330,7 +336,7 @@ def prefixed_warnings(prefix: str) -> Generator[None, None, None]:
 
 class LogCollector:
     def __init__(self) -> None:
-        self.logs: List[logging.LogRecord] = []
+        self.logs: list[logging.LogRecord] = []
 
     @contextmanager
     def collect(self) -> Generator[None, None, None]:
@@ -344,18 +350,15 @@ class InfoFilter(logging.Filter):
     """Filter error and warning messages."""
 
     def filter(self, record: logging.LogRecord) -> bool:
-        if record.levelno < logging.WARNING:
-            return True
-        else:
-            return False
+        return record.levelno < logging.WARNING
 
 
-def is_suppressed_warning(type: str, subtype: str, suppress_warnings: List[str]) -> bool:
-    """Check the warning is suppressed or not."""
+def is_suppressed_warning(type: str, subtype: str, suppress_warnings: list[str]) -> bool:
+    """Check whether the warning is suppressed or not."""
     if type is None:
         return False
 
-    subtarget: Optional[str]
+    subtarget: str | None
 
     for warning_type in suppress_warnings:
         if '.' in warning_type:
@@ -363,10 +366,8 @@ def is_suppressed_warning(type: str, subtype: str, suppress_warnings: List[str])
         else:
             target, subtarget = warning_type, None
 
-        if target == type:
-            if (subtype is None or subtarget is None or
-               subtarget == subtype or subtarget == '*'):
-                return True
+        if target == type and subtarget in (None, subtype, "*"):
+            return True
 
     return False
 
@@ -374,13 +375,13 @@ def is_suppressed_warning(type: str, subtype: str, suppress_warnings: List[str])
 class WarningSuppressor(logging.Filter):
     """Filter logs by `suppress_warnings`."""
 
-    def __init__(self, app: "Sphinx") -> None:
+    def __init__(self, app: Sphinx) -> None:
         self.app = app
         super().__init__()
 
     def filter(self, record: logging.LogRecord) -> bool:
-        type = getattr(record, 'type', None)
-        subtype = getattr(record, 'subtype', None)
+        type = getattr(record, 'type', '')
+        subtype = getattr(record, 'subtype', '')
 
         try:
             suppress_warnings = self.app.config.suppress_warnings
@@ -398,7 +399,7 @@ class WarningSuppressor(logging.Filter):
 class WarningIsErrorFilter(logging.Filter):
     """Raise exception if warning emitted."""
 
-    def __init__(self, app: "Sphinx") -> None:
+    def __init__(self, app: Sphinx) -> None:
         self.app = app
         super().__init__()
 
@@ -429,12 +430,12 @@ class DisableWarningIsErrorFilter(logging.Filter):
     """Disable WarningIsErrorFilter if this filter installed."""
 
     def filter(self, record: logging.LogRecord) -> bool:
-        record.skip_warningsiserror = True  # type: ignore
+        record.skip_warningsiserror = True
         return True
 
 
 class MessagePrefixFilter(logging.Filter):
-    """Prepend prefix to all records."""
+    """Prepend prefix to all log records."""
 
     def __init__(self, prefix: str) -> None:
         self.prefix = prefix
@@ -451,7 +452,7 @@ class OnceFilter(logging.Filter):
 
     def __init__(self, name: str = '') -> None:
         super().__init__(name)
-        self.messages: Dict[str, List] = {}
+        self.messages: dict[str, list] = {}
 
     def filter(self, record: logging.LogRecord) -> bool:
         once = getattr(record, 'once', '')
@@ -472,9 +473,9 @@ class SphinxLogRecordTranslator(logging.Filter):
     * Make a instance of SphinxLogRecord
     * docname to path if location given
     """
-    LogRecordClass: Type[logging.LogRecord]
+    LogRecordClass: type[logging.LogRecord]
 
-    def __init__(self, app: "Sphinx") -> None:
+    def __init__(self, app: Sphinx) -> None:
         self.app = app
         super().__init__()
 
@@ -486,16 +487,17 @@ class SphinxLogRecordTranslator(logging.Filter):
         location = getattr(record, 'location', None)
         if isinstance(location, tuple):
             docname, lineno = location
-            if docname and lineno:
-                record.location = '%s:%s' % (self.app.env.doc2path(docname), lineno)
-            elif docname:
-                record.location = '%s' % self.app.env.doc2path(docname)
+            if docname:
+                if lineno:
+                    record.location = f'{self.app.env.doc2path(docname)}:{lineno}'
+                else:
+                    record.location = f'{self.app.env.doc2path(docname)}'
             else:
                 record.location = None
         elif isinstance(location, nodes.Node):
             record.location = get_node_location(location)
         elif location and ':' not in location:
-            record.location = '%s' % self.app.env.doc2path(location)
+            record.location = f'{self.app.env.doc2path(location)}'
 
         return True
 
@@ -510,16 +512,17 @@ class WarningLogRecordTranslator(SphinxLogRecordTranslator):
     LogRecordClass = SphinxWarningLogRecord
 
 
-def get_node_location(node: Node) -> Optional[str]:
-    (source, line) = get_source_line(node)
+def get_node_location(node: Node) -> str | None:
+    source, line = get_source_line(node)
+    if source:
+        source = abspath(source)
     if source and line:
-        return "%s:%s" % (source, line)
-    elif source:
-        return "%s:" % source
-    elif line:
-        return "<unknown>:%s" % line
-    else:
-        return None
+        return f"{source}:{line}"
+    if source:
+        return f"{source}:"
+    if line:
+        return f"<unknown>:{line}"
+    return None
 
 
 class ColorizeFormatter(logging.Formatter):
@@ -555,15 +558,15 @@ class SafeEncodingWriter:
 
 
 class LastMessagesWriter:
-    """Stream writer which memories last 10 messages to save trackback"""
-    def __init__(self, app: "Sphinx", stream: IO) -> None:
+    """Stream writer storing last 10 messages in memory to save trackback"""
+    def __init__(self, app: Sphinx, stream: IO) -> None:
         self.app = app
 
     def write(self, data: str) -> None:
         self.app.messagelog.append(data)
 
 
-def setup(app: "Sphinx", status: IO, warning: IO) -> None:
+def setup(app: Sphinx, status: IO, warning: IO) -> None:
     """Setup root logger for Sphinx"""
     logger = logging.getLogger(NAMESPACE)
     logger.setLevel(logging.DEBUG)
