@@ -1,17 +1,11 @@
-"""
-    sphinx.util.rst
-    ~~~~~~~~~~~~~~~
+"""reST helper functions."""
 
-    reST helper functions.
-
-    :copyright: Copyright 2007-2021 by the Sphinx team, see AUTHORS.
-    :license: BSD, see LICENSE for details.
-"""
+from __future__ import annotations
 
 import re
 from collections import defaultdict
 from contextlib import contextmanager
-from typing import Dict, Generator
+from typing import Generator
 from unicodedata import east_asian_width
 
 from docutils.parsers.rst import roles
@@ -24,7 +18,7 @@ from sphinx.locale import __
 from sphinx.util import docutils, logging
 
 try:
-    from jinja2.utils import pass_environment  # type: ignore  # jinja2-3.0 or above
+    from jinja2.utils import pass_environment
 except ImportError:
     from jinja2 import environmentfilter as pass_environment
 
@@ -36,7 +30,7 @@ symbols_re = re.compile(r'([!-\-/:-@\[-`{-~])')  # symbols without dot(0x2e)
 SECTIONING_CHARS = ['=', '-', '~']
 
 # width of characters
-WIDECHARS: Dict[str, str] = defaultdict(lambda: "WF")  # WF: Wide + Full-width
+WIDECHARS: dict[str, str] = defaultdict(lambda: "WF")  # WF: Wide + Full-width
 WIDECHARS["ja"] = "WFA"  # In Japanese, Ambiguous characters also have double width
 
 
@@ -61,9 +55,9 @@ def textwidth(text: str, widechars: str = 'WF') -> int:
 def heading(env: Environment, text: str, level: int = 1) -> str:
     """Create a heading for *level*."""
     assert level <= 3
-    width = textwidth(text, WIDECHARS[env.language])  # type: ignore
+    width = textwidth(text, WIDECHARS[env.language])
     sectioning_char = SECTIONING_CHARS[level - 1]
-    return '%s\n%s' % (text, sectioning_char * width)
+    return f'{text}\n{sectioning_char * width}'
 
 
 @contextmanager
@@ -71,7 +65,7 @@ def default_role(docname: str, name: str) -> Generator[None, None, None]:
     if name:
         dummy_reporter = Reporter('', 4, 4)
         role_fn, _ = roles.role(name, english, 0, dummy_reporter)
-        if role_fn:
+        if role_fn:  # type: ignore[truthy-function]
             docutils.register_role('', role_fn)
         else:
             logger.warning(__('default role %s not found'), name, location=docname)
@@ -106,7 +100,7 @@ def prepend_prolog(content: StringList, prolog: str) -> None:
 def append_epilog(content: StringList, epilog: str) -> None:
     """Append a string to content body as epilog."""
     if epilog:
-        if 0 < len(content):
+        if len(content) > 0:
             source, lineno = content.info(-1)
         else:
             source = '<generated>'
